
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import numpy as np
import scipy.ndimage as spImg
import math




def preprocess(image):
    
    # Tiefpassfilterung:
    result = spImg.gaussian_filter(image, 1)
    
    # Binaerisieren:
    m = np.mean(result)
    result = np.array([[ (0 if v < m else 1) for v in row] for row in result], dtype="int")
    
    # Sobel:
    gx = spImg.filters.sobel(result, 1)
    gy = spImg.filters.sobel(result, 0)  
    result = np.sqrt(gx ** 2 + gx ** 2)
    
    # Binaerisieren:
    m = np.mean(result)
    result = np.array([[ (0 if v > m else 1) for v in row] for row in result], dtype="int")
     
    return result




def houghTransformAt(houghDomain, cVals, sVals, xc, yc, radius):
    
    h, w = houghDomain.shape
    for angle in xrange(0, 360, 5):
        x = int(xc + cVals[angle]) #int(xc + radius * math.cos(a))
        y = int(yc + sVals[angle]) #int(yc + radius * math.sin(a))
        if(x >= 0 and x < w and y >= 0 and y < h):
            houghDomain[y, x] += 1


def houghTransform(image, radius):
  
    # Hough-Raum diskretisiert "anlegen":
    h, w = image.shape
    houghDomain = np.zeros_like(image)
    
    
    cVals = np.zeros((360))
    sVals = np.zeros((360))
    for angle in xrange(0, 360, 1):
        a = math.radians(angle)
        cVals[angle] = radius * math.cos(a)
        sVals[angle] = radius * math.sin(a)
              
    for y in xrange(h):
        print y
        for x in xrange(w):
            if(image[y, x] == 0):
                houghTransformAt(houghDomain, cVals, sVals, x, y, radius)
                
    return houghDomain
    
 
 
 
def findCoins(houghDomain, raster=8):
    
    coinPositions = []
    
    h, w = houghDomain.shape 
    lastX = 0
    lastY = 0
    for y in xrange(h / raster, h + h / raster, h / raster):
        for x in xrange(w / raster, w + w / raster, w / raster):
            
            part = houghDomain[lastY:y, lastX:x]
            xc = -1
            yc = -1
            maxVal = np.max(part)
            if(maxVal > 0):
                for i in xrange(part.shape[0]):
                    if(xc >= 0): break
                    for j in xrange(part.shape[1]):
                        if(part[i, j] == maxVal):
                            xc = lastX + j
                            yc = lastY + i
                            break          
            if(xc >= 0):  
                print xc, yc  
                coinPositions.append((xc, yc))         
            lastX = x
        lastY = y
        lastX = 0
        
    return np.array(coinPositions)



def markCoins(coinPositions, radius, color):
    
    plt.scatter(coinPositions[:, 0], coinPositions[:, 1], color=color)
    
    for coinPos in coinPositions:
        x, y = coinPos
        c = plt.Circle((x, y), radius=radius, color=color, fill=False)
        plt.gca().add_artist(c)
    



def findAndMarkCoins(image, radius, threshold = 0.75):
    
    result = houghTransform(image, radius)

    resultThreshold = result.copy()
    resultThreshold[ resultThreshold < threshold * np.max(resultThreshold)] = 0
    
    coinPositions = findCoins(resultThreshold)
    return result, coinPositions, radius


def aufgabe1b():
    
    input = plt.imread('coins_s.png')
    preprocessed = preprocess(input)
    
    houghDomainImage, coinPositions, radius = findAndMarkCoins(preprocessed, 26)
    houghDomainImage2, coinPositions2, radius2 = findAndMarkCoins(preprocessed, 29)
    
    
    plt.figure(1)
    plt.subplot(321)
    plt.imshow(input, cmap=plt.cm.gray)
    plt.subplot(322)
    plt.imshow(preprocessed, cmap=plt.cm.gray)
    plt.subplot(323)
    plt.imshow(houghDomainImage, cmap=plt.cm.gray)
    plt.subplot(324)
    plt.imshow(houghDomainImage2, cmap=plt.cm.gray)
    plt.subplot(325)
    plt.imshow(input, cmap=plt.cm.gray)
    markCoins(coinPositions, radius, "r")
    markCoins(coinPositions2, radius2, "b")
    plt.show()
    
    

aufgabe1b()
